__all__ = ['jdump', 'jload', 'clean_path', 'parse_line', 'duration_tidy', 'DEBUG']

import traceback
import itertools
import string
import sys
import os
from os.path import expanduser, expandvars, abspath
import logging

log = logging.getLogger('zicbee')
log.addHandler(logging.FileHandler('/tmp/zicbee.log'))
if 'DEBUG' in os.environ:
    try:
        val = logging.ERROR - int(os.environ['DEBUG'])*10
    except ValueError:
        val = logging.DEBUG

    log.setLevel(val)

def DEBUG():
    traceback.print_stack()
    traceback.print_exc()

# Filename path cleaner
def clean_path(path):
    return expanduser(abspath(expandvars(path)))

# int (de)compacter [int <> small str convertors]
# convert to base62...
base = 62
chars = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z']

def compact_int(ival):
    result = []
    rest = ival
    b = base
    while True:
        int_part, rest = divmod(rest, b)
        result.append(chars[rest])
        if not int_part:
            break
        rest = int_part
    return "".join(reversed(result))

def uncompact_int(str_val):
    # int(x, base) not used because it's limited to base 36
    unit = 1
    result = 0
    for char in reversed(str_val):
        result += chars.index(char) * unit
        unit *= base
    return result

################################################################################

#
# Try to get the most performant json backend
#
# cjson:
# 10 loops, best of 3: 226 msec per loop
# simplejson:
# 1 loops, best of 3: 10.3 sec per loop
# demjson:
# 1 loops, best of 3: 65.2 sec per loop
#

json_engine = None
try:
    from cjson import encode as jdump, decode as jload
    json_engine = 'cjson'
except ImportError:
    try:
        from simplejson import dumps as jdump, loads as jload
        json_engine = 'simplejson'
    except ImportError:
        from demjson import encode as jdump, decode as jload
        json_engine = 'demjson'

sys.stderr.write("using %s.\n"%json_engine)
################################################################################

_plur = lambda val: 's' if val > 1 else ''

def duration_tidy(orig):
    minutes, seconds = divmod(orig, 60)
    if minutes > 60:
        hours, minutes = divmod(minutes, 60)
        if hours > 24:
            days, hours = divmod(hours, 24)
            return '%d day%s, %d hour%s %d min %02.1fs.'%(days, _plur(days), hours, _plur(hours), minutes, seconds)
        else:
            return '%d hour%s %d min %02.1fs.'%(hours, 's' if hours>1 else '', minutes, seconds)
    else:
        return '%d min %02.1fs.'%(minutes, seconds)
    if minutes > 60:
        hours = int(minutes/60)
        minutes -= hours*60
        if hours > 24:
            days = int(hours/24)
            hours -= days*24
            return '%d days, %d:%d.%ds.'%(days, hours, minutes, seconds)
        return '%d:%d.%ds.'%(hours, minutes, seconds)
    return '%d.%02ds.'%(minutes, seconds)

################################################################################
# line parser

properties = []
for name in 'score tags length artist title album filename'.split():
    properties.append(name)
    properties.append(name.title())
del name

def _find_property(line):
    tab = line.rsplit(None, 1)
    for prop in properties:
        if prop == tab[-1]:
            if len(tab) == 1:
                return line
            else:
                return tab[-1], tab[0].strip()

def _conv_line(txt):
    # TODO: replace with a real parser ?
    split_line = txt.split(':')
    log.debug('split line: %s'% split_line)

    if len(split_line) > 1:
        ret = []
        attr = None
        for elt in split_line:
            if attr is None:
                if elt[0] == '(':
                    elt = elt[1:].strip()
                    ret.append('(')
                attr = elt
                log.debug('attr: %s'%elt)
            else:
                props = _find_property(elt)
                log.debug('props: %s'%repr(props))
                if props:
                    # get init vars
                    name, val = props
                    vals = val.rsplit(None, 1)
                    # set defaults
                    bin_operator = 'and'
                    next_is_grouped = False
                    end_of_group = False
                    if len(vals) > 1: # check the last element
                        if vals[1] == '(':
                            vals = vals[0].rsplit(None, 1)
                            next_is_grouped = True
                        if vals[1] in ('or', 'and', 'or!', 'and!'):
                            val = vals[0]
                            if val[-1] == ')':
                                val = val[:-1].strip()
                                end_of_group = True
                            bin_operator = vals[1]
                            if bin_operator[-1] == '!':
                                bin_operator = bin_operator[:-1] + ' not'
                    log.debug('ret.append(%r, %r)', attr, val)
                    ret.append( (attr, val) )
                    if end_of_group:
                        ret.append(')')
                    log.debug('ret.append(%r)', bin_operator)
                    ret.append( bin_operator )
                    attr = name
                    if next_is_grouped:
                        ret.append('(')
                else:
                    elt = elt.strip()
                    end_of_group = False
                    if elt[-1] == ')':
                        elt = elt[:-1].strip()
                        end_of_group = True
                    log.debug('noprops: %r / %r', attr, repr(elt))
                    ret.append( (attr, elt.strip()) )
                    if end_of_group:
                        ret.append(')')
        log.debug('ret: %s'%txt)
        return ret
    else:
        log.debug('txt: %s'%txt)
        return txt

RAW_ATTRS = ('filename',)

def parse_line(line):
    ret = _conv_line(line)
    log.debug('RET: %s'%repr(ret))
    # string (simple) handling
    if isinstance(ret, basestring):
        if ret:
            return ('%(t)s in artist or %(t)s in title or %(t)s in filename or %(t)s in album'%dict(t='"%s"'%ret.replace('"', r'\"')), {})
        else:
            return ('1', {})
    # complex search
    varnames = list(string.ascii_letters)
    args = {}
    str_list = []

    def try_dec(txt):
        try:
            return txt.decode('utf-8')
        except UnicodeDecodeError:
            return txt.decode('latin1')

    for pattern in ret:
        if isinstance(pattern, basestring):
            log.debug('str_list.append("%s")', pattern)
            str_list.append(pattern)
        else:
            attr_name, value = pattern
            var_name = varnames.pop(0)
            if attr_name in ('length', 'score'):
                # numeric
                modifier = ''
                while value[0] in '>=<':
                    modifier += value[0]
                    value = value[1:]

                log.debug('str_list.append("%r %r %r")', attr_name, modifier or '==', var_name)
                str_list.append('%s %s %s'%(attr_name, modifier or '==', var_name))
                args[var_name] = eval(value)
            else:
                if attr_name in RAW_ATTRS:
                    value = str(value)

                # If attr name is capitalized, no case-unsensitive search
                if attr_name[0].islower():
                    value = value.lower()
                    attr_name += '.lower()'
                else:
                    attr_name = attr_name.lower()
                str_list.append('%s in %s'%(try_dec(repr(value)), attr_name))
                log.debug('str_list.append(%s)'%str_list[-1])
    return ' '.join(str_list), args




# Access MPlayer from python
import os
import select
import subprocess

DEBUG='DEBUG' in os.environ

class MPlayer(object):
    ''' A class to access a slave mplayer process
    you may also want to use command(name, args*) directly

    Exemples:
        mp.command('loadfile', '/desktop/funny.mp3')
        mp.command('pause')
        mp.command('quit')

    Or:
        mp.loadfile('/desktop/funny.mp3')
        mp.pause()
        mp.quit()
    '''

    exe_name = 'mplayer' if os.sep == '/' else 'mplayer.exe'

    def __init__(self):
        self._spawn()

    def wait(self):
        self._mplayer.wait()

    def _spawn(self):
        self._mplayer = subprocess.Popen(
                [self.exe_name, '-cache', '128', '-slave', '-quiet', '-idle'],
                stdin=subprocess.PIPE, stdout=subprocess.PIPE, bufsize=1)
        self._readlines()

    def __del__(self):
        self._mplayer.stdin.write('quit\n')

    def _readlines(self, timeout=0.4):
        ret = []
        while any(select.select([self._mplayer.stdout.fileno()], [], [], timeout)):
            ret.append( self._mplayer.stdout.readline() )
        return ret

    def _get_meta(self):
        try:
            meta = self.prop_metadata.split(',')
        except AttributeError:
            return None
        else:
            return dict(zip(meta[::2], meta[1::2]))

    meta = property(_get_meta, doc="Get metadatas as a dict"); del _get_meta

    def command(self, name, *args):
        ''' Very basic interface
        Sends command 'name' to process, with given args
        '''
        ret = self._readlines(0.01) # Flush
        if DEBUG:
            print "FLUSH LINES:", ret
        cmd = '%s%s%s\n'%(name,
                ' ' if args else '',
                ' '.join(repr(a) for a in args)
                )
        if DEBUG:
            print "CMD:", cmd
        try:
            self._mplayer.stdin.write(cmd)
        except IOError:
            self._spawn()
            self._mplayer.stdin.write(cmd)

        if name == 'quit':
            return
        ret = self._readlines()
        if DEBUG:
            print "READ LINES:", ret

        if not ret:
            return None
        else:
            ret = ret[-1]

        if ret.startswith('ANS'):
            val = ret.split('=', 1)[1].rstrip()
            try:
                return eval(val)
            except:
                return val
        return ret

    def radio_step_channel(self, *args):
        """ radio_step_channel <-1|1>
    Step forwards (1) or backwards (-1) in channel list. Works only when the
    'channels' radio parameter was set.

        """
        if len(args) != 1:
            raise TypeError('radio_step_channel takes 1 arguments (%d given)'%len(args))
        return self.command('radio_step_channel', *args)

    def radio_set_channel(self, *args):
        """ radio_set_channel <channel>
    Switch to <channel>. The 'channels' radio parameter needs to be set.

        """
        if len(args) != 1:
            raise TypeError('radio_set_channel takes 1 arguments (%d given)'%len(args))
        return self.command('radio_set_channel', *args)

    def radio_set_freq(self, *args):
        """ radio_set_freq <frequency in MHz>
    Set the radio tuner frequency.

        """
        if len(args) != 1:
            raise TypeError('radio_set_freq takes 1 arguments (%d given)'%len(args))
        return self.command('radio_set_freq', *args)

    def radio_step_freq(self, *args):
        """ radio_step_freq <value>
    Tune frequency by the <value> (positive - up, negative - down). 

        """
        if len(args) != 1:
            raise TypeError('radio_step_freq takes 1 arguments (%d given)'%len(args))
        return self.command('radio_step_freq', *args)

    def seek(self, *args):
        """ seek <value> [type]
    Seek to some place in the movie.
        0 is a relative seek of +/- <value> seconds (default).
        1 is a seek to <value> % in the movie.
        2 is a seek to an absolute position of <value> seconds.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('seek takes 2 arguments (%d given)'%len(args))
        return self.command('seek', *args)

    def edl_mark(self, *args):
        """ edl_mark
    Write the current position into the EDL file.

        """
        if len(args) != 0:
            raise TypeError('edl_mark takes 0 arguments (%d given)'%len(args))
        return self.command('edl_mark', *args)

    def audio_delay(self, *args):
        """ audio_delay <value> [abs]
    Set/adjust the audio delay.
    If [abs] is not given or is zero, adjust the delay by <value> seconds.
    If [abs] is nonzero, set the delay to <value> seconds.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('audio_delay takes 2 arguments (%d given)'%len(args))
        return self.command('audio_delay', *args)

    def speed_incr(self, *args):
        """ speed_incr <value>
    Add <value> to the current playback speed.

        """
        if len(args) != 1:
            raise TypeError('speed_incr takes 1 arguments (%d given)'%len(args))
        return self.command('speed_incr', *args)

    def speed_mult(self, *args):
        """ speed_mult <value>
    Multiply the current speed by <value>.

        """
        if len(args) != 1:
            raise TypeError('speed_mult takes 1 arguments (%d given)'%len(args))
        return self.command('speed_mult', *args)

    def speed_set(self, *args):
        """ speed_set <value>
    Set the speed to <value>.

        """
        if len(args) != 1:
            raise TypeError('speed_set takes 1 arguments (%d given)'%len(args))
        return self.command('speed_set', *args)

    def quit(self, *args):
        """ quit [value]
    Quit MPlayer. The optional integer [value] is used as the return code
    for the mplayer process (default: 0).

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('quit takes 1 arguments (%d given)'%len(args))
        return self.command('quit', *args)

    def pause(self, *args):
        """ pause
    Pause/unpause the playback.

        """
        if len(args) != 0:
            raise TypeError('pause takes 0 arguments (%d given)'%len(args))
        return self.command('pause', *args)

    def frame_step(self, *args):
        """ frame_step
    Play one frame, then pause again.

        """
        if len(args) != 0:
            raise TypeError('frame_step takes 0 arguments (%d given)'%len(args))
        return self.command('frame_step', *args)

    def pt_step(self, *args):
        """ pt_step <value> [force]
    Go to the next/previous entry in the playtree. The sign of <value> tells
    the direction.  If no entry is available in the given direction it will do
    nothing unless [force] is non-zero.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('pt_step takes 2 arguments (%d given)'%len(args))
        return self.command('pt_step', *args)

    def pt_up_step(self, *args):
        """ pt_up_step <value> [force]
    Similar to pt_step but jumps to the next/previous entry in the parent list.
    Useful to break out of the inner loop in the playtree.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('pt_up_step takes 2 arguments (%d given)'%len(args))
        return self.command('pt_up_step', *args)

    def alt_src_step(self, *args):
        """ alt_src_step <value> (ASX playlist only)
    When more than one source is available it selects the next/previous one.

        """
        if len(args) != 1:
            raise TypeError('alt_src_step takes 1 arguments (%d given)'%len(args))
        return self.command('alt_src_step', *args)

    def loop(self, *args):
        """ loop <value> [abs]
    Adjust/set how many times the movie should be looped. -1 means no loop,
    and 0 forever.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('loop takes 2 arguments (%d given)'%len(args))
        return self.command('loop', *args)

    def sub_delay(self, *args):
        """ sub_delay <value> [abs]
    Adjust the subtitle delay by +/- <value> seconds or set it to <value>
    seconds when [abs] is nonzero.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('sub_delay takes 2 arguments (%d given)'%len(args))
        return self.command('sub_delay', *args)

    def sub_step(self, *args):
        """ sub_step <value>
    Step forward in the subtitle list by <value> steps or backwards if <value>
    is negative.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('sub_step takes 2 arguments (%d given)'%len(args))
        return self.command('sub_step', *args)

    def osd(self, *args):
        """ osd [level]
    Toggle OSD mode or set it to [level] when [level] >= 0.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('osd takes 1 arguments (%d given)'%len(args))
        return self.command('osd', *args)

    def osd_show_text(self, *args):
        """ osd_show_text <string> [duration] [level]
    Show <string> on the OSD.

        """
        if not (1 <= len(args) <= 3):
            raise TypeError('osd_show_text takes 3 arguments (%d given)'%len(args))
        return self.command('osd_show_text', *args)

    def osd_show_property_te(self, *args):
        """ osd_show_property_text <string> [duration] [level]
    Show an expanded property string on the OSD, see -playing-msg for a
    description of the available expansions. If [duration] is >= 0 the text
    is shown for [duration] ms. [level] sets the minimum OSD level needed
    for the message to be visible (default: 0 - always show).

        """
        if not (1 <= len(args) <= 3):
            raise TypeError('osd_show_property_te takes 3 arguments (%d given)'%len(args))
        return self.command('osd_show_property_te', *args)

    def volume(self, *args):
        """ volume <value> [abs]
    Increase/decrease volume or set it to <value> if [abs] is nonzero.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('volume takes 2 arguments (%d given)'%len(args))
        return self.command('volume', *args)

    def balance(self, *args):
        """ balance(float, integer=None)
        """
        if not (1 <= len(args) <= 2):
            raise TypeError('balance takes 2 arguments (%d given)'%len(args))
        return self.command('balance', *args)

    def use_master(self, *args):
        """ use_master
    Switch volume control between master and PCM.

        """
        if len(args) != 0:
            raise TypeError('use_master takes 0 arguments (%d given)'%len(args))
        return self.command('use_master', *args)

    def mute(self, *args):
        """ mute [value]
    Toggle sound output muting or set it to [value] when [value] >= 0
    (1 == on, 0 == off).

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('mute takes 1 arguments (%d given)'%len(args))
        return self.command('mute', *args)

    def contrast(self, *args):
        """ contrast(integer, integer=None)
        """
        if not (1 <= len(args) <= 2):
            raise TypeError('contrast takes 2 arguments (%d given)'%len(args))
        return self.command('contrast', *args)

    def gamma(self, *args):
        """ gamma(integer, integer=None)
        """
        if not (1 <= len(args) <= 2):
            raise TypeError('gamma takes 2 arguments (%d given)'%len(args))
        return self.command('gamma', *args)

    def brightness(self, *args):
        """ brightness(integer, integer=None)
        """
        if not (1 <= len(args) <= 2):
            raise TypeError('brightness takes 2 arguments (%d given)'%len(args))
        return self.command('brightness', *args)

    def hue(self, *args):
        """ hue(integer, integer=None)
        """
        if not (1 <= len(args) <= 2):
            raise TypeError('hue takes 2 arguments (%d given)'%len(args))
        return self.command('hue', *args)

    def saturation(self, *args):
        """ saturation(integer, integer=None)
        """
        if not (1 <= len(args) <= 2):
            raise TypeError('saturation takes 2 arguments (%d given)'%len(args))
        return self.command('saturation', *args)

    def frame_drop(self, *args):
        """ frame_drop [value]
    Toggle/set frame dropping mode.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('frame_drop takes 1 arguments (%d given)'%len(args))
        return self.command('frame_drop', *args)

    def sub_pos(self, *args):
        """ sub_pos <value> [abs]
    Adjust/set subtitle position.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('sub_pos takes 2 arguments (%d given)'%len(args))
        return self.command('sub_pos', *args)

    def sub_alignment(self, *args):
        """ sub_alignment [value]
    Toggle/set subtitle alignment.
        0 top alignment
        1 center alignment
        2 bottom alignment

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('sub_alignment takes 1 arguments (%d given)'%len(args))
        return self.command('sub_alignment', *args)

    def sub_visibility(self, *args):
        """ sub_visibility [value]
    Toggle/set subtitle visibility.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('sub_visibility takes 1 arguments (%d given)'%len(args))
        return self.command('sub_visibility', *args)

    def sub_load(self, *args):
        """ sub_load <subtitle_file>
    Loads subtitles from <subtitle_file>.

        """
        if len(args) != 1:
            raise TypeError('sub_load takes 1 arguments (%d given)'%len(args))
        return self.command('sub_load', *args)

    def sub_remove(self, *args):
        """ sub_remove [value]
    If the [value] argument is present and non-negative, removes the subtitle
    file with index [value]. If the argument is omitted or negative, removes
    all subtitle files.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('sub_remove takes 1 arguments (%d given)'%len(args))
        return self.command('sub_remove', *args)

    def vobsub_lang(self, *args):
        """ vobsub_lang
    This is a stub linked to sub_select for backwards compatibility.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('vobsub_lang takes 1 arguments (%d given)'%len(args))
        return self.command('vobsub_lang', *args)

    def sub_select(self, *args):
        """ sub_select [value]
    Display subtitle with index [value]. Turn subtitle display off if
    [value] is -1 or greater than the highest available subtitle index.
    Cycle through the available subtitles if [value] is omitted or less
    than -1. Supported subtitle sources are -sub options on the command
    line, VOBsubs, DVD subtitles, and Ogg and Matroska text streams.
    This command is mainly for cycling all subtitles, if you want to set
    a specific subtitle, use sub_file, sub_vob, or sub_demux.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('sub_select takes 1 arguments (%d given)'%len(args))
        return self.command('sub_select', *args)

    def sub_log(self, *args):
        """ sub_log
    Logs the current or last displayed subtitle together with filename
    and time information to ~/.mplayer/subtitle_log. Intended purpose
    is to allow convenient marking of bogus subtitles which need to be
    fixed while watching the movie.

        """
        if len(args) != 0:
            raise TypeError('sub_log takes 0 arguments (%d given)'%len(args))
        return self.command('sub_log', *args)

    def sub_scale(self, *args):
        """ sub_scale <value> [abs]
    Adjust the subtitle size by +/- <value> or set it to <value> when [abs]
    is nonzero.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('sub_scale takes 2 arguments (%d given)'%len(args))
        return self.command('sub_scale', *args)

    def get_percent_pos(self, *args):
        """ get_percent_pos
    Print out the current position in the file, as integer percentage [0-100).

        """
        if len(args) != 0:
            raise TypeError('get_percent_pos takes 0 arguments (%d given)'%len(args))
        return self.command('get_percent_pos', *args)

    def get_time_pos(self, *args):
        """ get_time_pos
    Print out the current position in the file in seconds, as float.

        """
        if len(args) != 0:
            raise TypeError('get_time_pos takes 0 arguments (%d given)'%len(args))
        return self.command('get_time_pos', *args)

    def get_time_length(self, *args):
        """ get_time_length
    Print out the length of the current file in seconds.

        """
        if len(args) != 0:
            raise TypeError('get_time_length takes 0 arguments (%d given)'%len(args))
        return self.command('get_time_length', *args)

    def get_file_name(self, *args):
        """ get_file_name
    Print out the name of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_file_name takes 0 arguments (%d given)'%len(args))
        return self.command('get_file_name', *args)

    def get_video_codec(self, *args):
        """ get_video_codec
    Print out the video codec name of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_video_codec takes 0 arguments (%d given)'%len(args))
        return self.command('get_video_codec', *args)

    def get_video_bitrate(self, *args):
        """ get_video_bitrate
    Print out the video bitrate of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_video_bitrate takes 0 arguments (%d given)'%len(args))
        return self.command('get_video_bitrate', *args)

    def get_video_resolution(self, *args):
        """ get_video_resolution
    Print out the video resolution of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_video_resolution takes 0 arguments (%d given)'%len(args))
        return self.command('get_video_resolution', *args)

    def get_audio_codec(self, *args):
        """ get_audio_codec
    Print out the audio codec name of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_audio_codec takes 0 arguments (%d given)'%len(args))
        return self.command('get_audio_codec', *args)

    def get_audio_bitrate(self, *args):
        """ get_audio_bitrate
    Print out the audio bitrate of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_audio_bitrate takes 0 arguments (%d given)'%len(args))
        return self.command('get_audio_bitrate', *args)

    def get_audio_samples(self, *args):
        """ get_audio_samples
    Print out the audio frequency and number of channels of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_audio_samples takes 0 arguments (%d given)'%len(args))
        return self.command('get_audio_samples', *args)

    def get_meta_title(self, *args):
        """ get_meta_title
    Print out the 'Title' metadata of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_meta_title takes 0 arguments (%d given)'%len(args))
        return self.command('get_meta_title', *args)

    def get_meta_artist(self, *args):
        """ get_meta_artist
    Print out the 'Artist' metadata of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_meta_artist takes 0 arguments (%d given)'%len(args))
        return self.command('get_meta_artist', *args)

    def get_meta_album(self, *args):
        """ get_meta_album
    Print out the 'Album' metadata of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_meta_album takes 0 arguments (%d given)'%len(args))
        return self.command('get_meta_album', *args)

    def get_meta_year(self, *args):
        """ get_meta_year
    Print out the 'Year' metadata of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_meta_year takes 0 arguments (%d given)'%len(args))
        return self.command('get_meta_year', *args)

    def get_meta_comment(self, *args):
        """ get_meta_comment
    Print out the 'Comment' metadata of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_meta_comment takes 0 arguments (%d given)'%len(args))
        return self.command('get_meta_comment', *args)

    def get_meta_track(self, *args):
        """ get_meta_track
    Print out the 'Track Number' metadata of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_meta_track takes 0 arguments (%d given)'%len(args))
        return self.command('get_meta_track', *args)

    def get_meta_genre(self, *args):
        """ get_meta_genre
    Print out the 'Genre' metadata of the current file.

        """
        if len(args) != 0:
            raise TypeError('get_meta_genre takes 0 arguments (%d given)'%len(args))
        return self.command('get_meta_genre', *args)

    def switch_audio(self, *args):
        """ switch_audio [value] (currently MPEG*, AVI, Matroska and streams handled by libavformat)
    Switch to the audio track with the ID [value]. Cycle through the
    available tracks if [value] is omitted or negative.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('switch_audio takes 1 arguments (%d given)'%len(args))
        return self.command('switch_audio', *args)

    def tv_start_scan(self, *args):
        """ tv_start_scan
    Start automatic TV channel scanning.

        """
        if len(args) != 0:
            raise TypeError('tv_start_scan takes 0 arguments (%d given)'%len(args))
        return self.command('tv_start_scan', *args)

    def tv_step_channel(self, *args):
        """ tv_step_channel <channel>
    Select next/previous TV channel.

        """
        if len(args) != 1:
            raise TypeError('tv_step_channel takes 1 arguments (%d given)'%len(args))
        return self.command('tv_step_channel', *args)

    def tv_step_norm(self, *args):
        """ tv_step_norm
    Change TV norm.

        """
        if len(args) != 0:
            raise TypeError('tv_step_norm takes 0 arguments (%d given)'%len(args))
        return self.command('tv_step_norm', *args)

    def tv_step_chanlist(self, *args):
        """ tv_step_chanlist
    Change channel list.

        """
        if len(args) != 0:
            raise TypeError('tv_step_chanlist takes 0 arguments (%d given)'%len(args))
        return self.command('tv_step_chanlist', *args)

    def tv_set_channel(self, *args):
        """ tv_set_channel <channel>
    Set the current TV channel.

        """
        if len(args) != 1:
            raise TypeError('tv_set_channel takes 1 arguments (%d given)'%len(args))
        return self.command('tv_set_channel', *args)

    def tv_last_channel(self, *args):
        """ tv_last_channel
    Set the current TV channel to the last one.

        """
        if len(args) != 0:
            raise TypeError('tv_last_channel takes 0 arguments (%d given)'%len(args))
        return self.command('tv_last_channel', *args)

    def tv_set_freq(self, *args):
        """ tv_set_freq <frequency in MHz>
    Set the TV tuner frequency.

        """
        if len(args) != 1:
            raise TypeError('tv_set_freq takes 1 arguments (%d given)'%len(args))
        return self.command('tv_set_freq', *args)

    def tv_step_freq(self, *args):
        """ tv_step_freq <frequency offset in MHz>
    Set the TV tuner frequency relative to current value.

        """
        if len(args) != 1:
            raise TypeError('tv_step_freq takes 1 arguments (%d given)'%len(args))
        return self.command('tv_step_freq', *args)

    def tv_set_norm(self, *args):
        """ tv_set_norm <norm>
    Set the TV tuner norm (PAL, SECAM, NTSC, ...).

        """
        if len(args) != 1:
            raise TypeError('tv_set_norm takes 1 arguments (%d given)'%len(args))
        return self.command('tv_set_norm', *args)

    def tv_set_brightness(self, *args):
        """ tv_set_brightness <-100 - 100> [abs]
    Set TV tuner brightness or adjust it if [abs] is set to 0.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('tv_set_brightness takes 2 arguments (%d given)'%len(args))
        return self.command('tv_set_brightness', *args)

    def tv_set_contrast(self, *args):
        """ tv_set_contrast <-100 -100> [abs]
    Set TV tuner contrast or adjust it if [abs] is set to 0.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('tv_set_contrast takes 2 arguments (%d given)'%len(args))
        return self.command('tv_set_contrast', *args)

    def tv_set_hue(self, *args):
        """ tv_set_hue <-100 - 100> [abs]
    Set TV tuner hue or adjust it if [abs] is set to 0.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('tv_set_hue takes 2 arguments (%d given)'%len(args))
        return self.command('tv_set_hue', *args)

    def tv_set_saturation(self, *args):
        """ tv_set_saturation <-100 - 100> [abs]
    Set TV tuner saturation or adjust it if [abs] is set to 0.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('tv_set_saturation takes 2 arguments (%d given)'%len(args))
        return self.command('tv_set_saturation', *args)

    def forced_subs_only(self, *args):
        """ forced_subs_only [value]
    Toggle/set forced subtitles only.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('forced_subs_only takes 1 arguments (%d given)'%len(args))
        return self.command('forced_subs_only', *args)

    def dvb_set_channel(self, *args):
        """ dvb_set_channel <channel_number> <card_number>
    Set DVB channel.

        """
        if len(args) != 2:
            raise TypeError('dvb_set_channel takes 2 arguments (%d given)'%len(args))
        return self.command('dvb_set_channel', *args)

    def switch_ratio(self, *args):
        """ switch_ratio [value]
    Change aspect ratio at runtime. [value] is the new aspect ratio expressed
    as a float (e.g. 1.77778 for 16/9).
    There might be problems with some video filters.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('switch_ratio takes 1 arguments (%d given)'%len(args))
        return self.command('switch_ratio', *args)

    def vo_fullscreen(self, *args):
        """ vo_fullscreen [value]
    Toggle/set fullscreen mode.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('vo_fullscreen takes 1 arguments (%d given)'%len(args))
        return self.command('vo_fullscreen', *args)

    def vo_ontop(self, *args):
        """ vo_ontop [value]
    Toggle/set stay-on-top.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('vo_ontop takes 1 arguments (%d given)'%len(args))
        return self.command('vo_ontop', *args)

    def file_filter(self, *args):
        """ file_filter(integer)
        """
        if len(args) != 1:
            raise TypeError('file_filter takes 1 arguments (%d given)'%len(args))
        return self.command('file_filter', *args)

    def vo_rootwin(self, *args):
        """ vo_rootwin [value]
    Toggle/set playback on the root window.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('vo_rootwin takes 1 arguments (%d given)'%len(args))
        return self.command('vo_rootwin', *args)

    def vo_border(self, *args):
        """ vo_border [value]
    Toggle/set borderless display.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('vo_border takes 1 arguments (%d given)'%len(args))
        return self.command('vo_border', *args)

    def screenshot(self, *args):
        """ screenshot <value>
    Take a screenshot. Requires the screenshot filter to be loaded.
        0 Take a single screenshot.
        1 Start/stop taking screenshot of each frame.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('screenshot takes 1 arguments (%d given)'%len(args))
        return self.command('screenshot', *args)

    def panscan(self, *args):
        """ panscan <-1.0 - 1.0> | <0.0 - 1.0> <abs>
    Increase or decrease the pan-and-scan range by <value>, 1.0 is the maximum.
    Negative values decrease the pan-and-scan range.
    If <abs> is != 0, then the pan-and scan range is interpreted as an
    absolute range.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('panscan takes 2 arguments (%d given)'%len(args))
        return self.command('panscan', *args)

    def switch_vsync(self, *args):
        """ switch_vsync [value]
    Toggle vsync (1 == on, 0 == off). If [value] is not provided,
    vsync status is inverted.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('switch_vsync takes 1 arguments (%d given)'%len(args))
        return self.command('switch_vsync', *args)

    def loadfile(self, *args):
        """ loadfile <file|url> <append>
    Load the given file/URL, stopping playback of the current file/URL.
    If <append> is nonzero playback continues and the file/URL is
    appended to the current playlist instead.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('loadfile takes 2 arguments (%d given)'%len(args))
        return self.command('loadfile', *args)

    def loadlist(self, *args):
        """ loadlist <file> <append>
    Load the given playlist file, stopping playback of the current file.
    If <append> is nonzero playback continues and the playlist file is
    appended to the current playlist instead.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('loadlist takes 2 arguments (%d given)'%len(args))
        return self.command('loadlist', *args)

    def run(self, *args):
        """ run <value>
    Run <value> as shell command. In OSD menu console mode stdout and stdin
    are through the video output driver.

        """
        if len(args) != 1:
            raise TypeError('run takes 1 arguments (%d given)'%len(args))
        return self.command('run', *args)

    def change_rectangle(self, *args):
        """ change_rectangle <val1> <val2>
    Change the position of the rectangle filter rectangle.
        <val1>
            Must be one of the following:
                0 = width
                1 = height
                2 = x position
                3 = y position
        <val2>
            If <val1> is 0 or 1:
                Integer amount to add/subtract from the width/height.
                Positive values add to width/height and negative values
                subtract from it.
            If <val1> is 2 or 3:
                Relative integer amount by which to move the upper left
                rectangle corner. Positive values move the rectangle
                right/down and negative values move the rectangle left/up.

        """
        if len(args) != 2:
            raise TypeError('change_rectangle takes 2 arguments (%d given)'%len(args))
        return self.command('change_rectangle', *args)

    def teletext_add_dec(self, *args):
        """ teletext_add_dec(string)
        """
        if len(args) != 1:
            raise TypeError('teletext_add_dec takes 1 arguments (%d given)'%len(args))
        return self.command('teletext_add_dec', *args)

    def teletext_go_link(self, *args):
        """ teletext_go_link <1-6>
    Follow given link on current teletext page.

        """
        if len(args) != 1:
            raise TypeError('teletext_go_link takes 1 arguments (%d given)'%len(args))
        return self.command('teletext_go_link', *args)

    def menu(self, *args):
        """ menu <command>
    Execute an OSD menu command.
        up     Move cursor up.
        down   Move cursor down.
        ok     Accept selection.
        cancel Cancel selection.
        hide   Hide the OSD menu.

        """
        if len(args) != 1:
            raise TypeError('menu takes 1 arguments (%d given)'%len(args))
        return self.command('menu', *args)

    def set_menu(self, *args):
        """ set_menu <menu_name>
    Display the menu named <menu_name>.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('set_menu takes 2 arguments (%d given)'%len(args))
        return self.command('set_menu', *args)

    def help(self, *args):
        """ help
    Displays help text, currently empty.

        """
        if len(args) != 0:
            raise TypeError('help takes 0 arguments (%d given)'%len(args))
        return self.command('help', *args)

    def exit(self, *args):
        """ exit
    Exits from OSD menu console. Unlike 'quit', does not quit MPlayer.

        """
        if len(args) != 0:
            raise TypeError('exit takes 0 arguments (%d given)'%len(args))
        return self.command('exit', *args)

    def hide(self, *args):
        """ hide
    Hides the OSD menu console. Clicking a menu command unhides it. Other
    keybindings act as usual.

        """
        if not (0 <= len(args) <= 1):
            raise TypeError('hide takes 1 arguments (%d given)'%len(args))
        return self.command('hide', *args)

    def get_vo_fullscreen(self, *args):
        """ get_vo_fullscreen
    Print out fullscreen status (1 == fullscreened, 0 == windowed).

        """
        if len(args) != 0:
            raise TypeError('get_vo_fullscreen takes 0 arguments (%d given)'%len(args))
        return self.command('get_vo_fullscreen', *args)

    def get_sub_visibility(self, *args):
        """ get_sub_visibility
    Print out subtitle visibility (1 == on, 0 == off).

        """
        if len(args) != 0:
            raise TypeError('get_sub_visibility takes 0 arguments (%d given)'%len(args))
        return self.command('get_sub_visibility', *args)

    def key_down_event(self, *args):
        """ key_down_event <value>
    Inject <value> key code event into MPlayer.

        """
        if len(args) != 1:
            raise TypeError('key_down_event takes 1 arguments (%d given)'%len(args))
        return self.command('key_down_event', *args)

    def set_property(self, *args):
        """ set_property <property> <value>
    Set a property.

        """
        if len(args) != 2:
            raise TypeError('set_property takes 2 arguments (%d given)'%len(args))
        return self.command('set_property', *args)

    def get_property(self, *args):
        """ get_property <property>
    Print out the current value of a property.

        """
        if len(args) != 1:
            raise TypeError('get_property takes 1 arguments (%d given)'%len(args))
        return self.command('get_property', *args)

    def step_property(self, *args):
        """ step_property <property> [value] [direction]
    Change a property by value, or increase by a default if value is
    not given or zero. The direction is reversed if direction is less
    than zero.

        """
        if not (1 <= len(args) <= 3):
            raise TypeError('step_property takes 3 arguments (%d given)'%len(args))
        return self.command('step_property', *args)

    def seek_chapter(self, *args):
        """ seek_chapter <value> [type]
    Seek to the start of a chapter.
        0 is a relative seek of +/- <value> chapters (default).
        1 is a seek to chapter <value>.

        """
        if not (1 <= len(args) <= 2):
            raise TypeError('seek_chapter takes 2 arguments (%d given)'%len(args))
        return self.command('seek_chapter', *args)

    def set_mouse_pos(self, *args):
        """ set_mouse_pos <x> <y>
    Tells MPlayer the coordinates of the mouse in the window.
    This command doesn't move the mouse!

        """
        if len(args) != 2:
            raise TypeError('set_mouse_pos takes 2 arguments (%d given)'%len(args))
        return self.command('set_mouse_pos', *args)

#Properties

    prop_osdlevel = property(
        lambda self: self.get_property("osdlevel"),
        lambda self, val: self.set_property("osdlevel", val),
        doc = '''as -osdlevel''')
        
    prop_speed = property(
        lambda self: self.get_property("speed"),
        lambda self, val: self.set_property("speed", val),
        doc = '''as -speed''')
        
    prop_loop = property(
        lambda self: self.get_property("loop"),
        lambda self, val: self.set_property("loop", val),
        doc = '''as -loop''')
        
    prop_pause = property(
        lambda self: self.get_property("pause"),
        None,
        doc = '''1 if paused, use with pausing_keep_force''')
        
    prop_filename = property(
        lambda self: self.get_property("filename"),
        None,
        doc = '''file playing wo path''')
        
    prop_path = property(
        lambda self: self.get_property("path"),
        None,
        doc = '''file playing''')
        
    prop_demuxer = property(
        lambda self: self.get_property("demuxer"),
        None,
        doc = '''demuxer used''')
        
    prop_stream_pos = property(
        lambda self: self.get_property("stream_pos"),
        lambda self, val: self.set_property("stream_pos", val),
        doc = '''position in stream''')
        
    prop_stream_start = property(
        lambda self: self.get_property("stream_start"),
        None,
        doc = '''start pos in stream''')
        
    prop_stream_end = property(
        lambda self: self.get_property("stream_end"),
        None,
        doc = '''end pos in stream''')
        
    prop_stream_length = property(
        lambda self: self.get_property("stream_length"),
        None,
        doc = '''(end - start)''')
        
    prop_chapter = property(
        lambda self: self.get_property("chapter"),
        lambda self, val: self.set_property("chapter", val),
        doc = '''select chapter''')
        
    prop_chapters = property(
        lambda self: self.get_property("chapters"),
        None,
        doc = '''number of chapters''')
        
    prop_angle = property(
        lambda self: self.get_property("angle"),
        lambda self, val: self.set_property("angle", val),
        doc = '''select angle''')
        
    prop_length = property(
        lambda self: self.get_property("length"),
        None,
        doc = '''length of file in seconds''')
        
    prop_percent_pos = property(
        lambda self: self.get_property("percent_pos"),
        lambda self, val: self.set_property("percent_pos", val),
        doc = '''position in percent''')
        
    prop_time_pos = property(
        lambda self: self.get_property("time_pos"),
        lambda self, val: self.set_property("time_pos", val),
        doc = '''position in seconds''')
        
    prop_metadata = property(
        lambda self: self.get_property("metadata"),
        None,
        doc = '''list of metadata key/value''')
        
    prop_volume = property(
        lambda self: self.get_property("volume"),
        lambda self, val: self.set_property("volume", val),
        doc = '''change volume''')
        
    prop_balance = property(
        lambda self: self.get_property("balance"),
        lambda self, val: self.set_property("balance", val),
        doc = '''change audio balance''')
        
    prop_mute = property(
        lambda self: self.get_property("mute"),
        lambda self, val: self.set_property("mute", val),
        doc = None)
        
    prop_audio_delay = property(
        lambda self: self.get_property("audio_delay"),
        lambda self, val: self.set_property("audio_delay", val),
        doc = None)
        
    prop_audio_format = property(
        lambda self: self.get_property("audio_format"),
        None,
        doc = None)
        
    prop_audio_codec = property(
        lambda self: self.get_property("audio_codec"),
        None,
        doc = None)
        
    prop_audio_bitrate = property(
        lambda self: self.get_property("audio_bitrate"),
        None,
        doc = None)
        
    prop_samplerate = property(
        lambda self: self.get_property("samplerate"),
        None,
        doc = None)
        
    prop_channels = property(
        lambda self: self.get_property("channels"),
        None,
        doc = None)
        
    prop_switch_audio = property(
        lambda self: self.get_property("switch_audio"),
        lambda self, val: self.set_property("switch_audio", val),
        doc = '''select audio stream''')
        
    prop_switch_angle = property(
        lambda self: self.get_property("switch_angle"),
        lambda self, val: self.set_property("switch_angle", val),
        doc = '''select DVD angle''')
        
    prop_switch_title = property(
        lambda self: self.get_property("switch_title"),
        lambda self, val: self.set_property("switch_title", val),
        doc = '''select DVD title''')
        
    prop_fullscreen = property(
        lambda self: self.get_property("fullscreen"),
        lambda self, val: self.set_property("fullscreen", val),
        doc = None)
        
    prop_deinterlace = property(
        lambda self: self.get_property("deinterlace"),
        lambda self, val: self.set_property("deinterlace", val),
        doc = None)
        
    prop_ontop = property(
        lambda self: self.get_property("ontop"),
        lambda self, val: self.set_property("ontop", val),
        doc = None)
        
    prop_rootwin = property(
        lambda self: self.get_property("rootwin"),
        lambda self, val: self.set_property("rootwin", val),
        doc = None)
        
    prop_border = property(
        lambda self: self.get_property("border"),
        lambda self, val: self.set_property("border", val),
        doc = None)
        
    prop_framedropping = property(
        lambda self: self.get_property("framedropping"),
        lambda self, val: self.set_property("framedropping", val),
        doc = '''1 = soft, 2 = hard''')
        
    prop_gamma = property(
        lambda self: self.get_property("gamma"),
        lambda self, val: self.set_property("gamma", val),
        doc = None)
        
    prop_brightness = property(
        lambda self: self.get_property("brightness"),
        lambda self, val: self.set_property("brightness", val),
        doc = None)
        
    prop_contrast = property(
        lambda self: self.get_property("contrast"),
        lambda self, val: self.set_property("contrast", val),
        doc = None)
        
    prop_saturation = property(
        lambda self: self.get_property("saturation"),
        lambda self, val: self.set_property("saturation", val),
        doc = None)
        
    prop_hue = property(
        lambda self: self.get_property("hue"),
        lambda self, val: self.set_property("hue", val),
        doc = None)
        
    prop_panscan = property(
        lambda self: self.get_property("panscan"),
        lambda self, val: self.set_property("panscan", val),
        doc = None)
        
    prop_vsync = property(
        lambda self: self.get_property("vsync"),
        lambda self, val: self.set_property("vsync", val),
        doc = None)
        
    prop_video_format = property(
        lambda self: self.get_property("video_format"),
        None,
        doc = None)
        
    prop_video_codec = property(
        lambda self: self.get_property("video_codec"),
        None,
        doc = None)
        
    prop_video_bitrate = property(
        lambda self: self.get_property("video_bitrate"),
        None,
        doc = None)
        
    prop_width = property(
        lambda self: self.get_property("width"),
        None,
        doc = '''"display" width''')
        
    prop_height = property(
        lambda self: self.get_property("height"),
        None,
        doc = '''"display" height''')
        
    prop_fps = property(
        lambda self: self.get_property("fps"),
        None,
        doc = None)
        
    prop_aspect = property(
        lambda self: self.get_property("aspect"),
        None,
        doc = None)
        
    prop_switch_video = property(
        lambda self: self.get_property("switch_video"),
        lambda self, val: self.set_property("switch_video", val),
        doc = '''select video stream''')
        
    prop_switch_program = property(
        lambda self: self.get_property("switch_program"),
        lambda self, val: self.set_property("switch_program", val),
        doc = '''(see TAB default keybind)''')
        
    prop_sub = property(
        lambda self: self.get_property("sub"),
        lambda self, val: self.set_property("sub", val),
        doc = '''select subtitle stream''')
        
    prop_sub_source = property(
        lambda self: self.get_property("sub_source"),
        lambda self, val: self.set_property("sub_source", val),
        doc = '''select subtitle source''')
        
    prop_sub_file = property(
        lambda self: self.get_property("sub_file"),
        lambda self, val: self.set_property("sub_file", val),
        doc = '''select file subtitles''')
        
    prop_sub_vob = property(
        lambda self: self.get_property("sub_vob"),
        lambda self, val: self.set_property("sub_vob", val),
        doc = '''select vobsubs''')
        
    prop_sub_demux = property(
        lambda self: self.get_property("sub_demux"),
        lambda self, val: self.set_property("sub_demux", val),
        doc = '''select subs from demux''')
        
    prop_sub_delay = property(
        lambda self: self.get_property("sub_delay"),
        lambda self, val: self.set_property("sub_delay", val),
        doc = None)
        
    prop_sub_pos = property(
        lambda self: self.get_property("sub_pos"),
        lambda self, val: self.set_property("sub_pos", val),
        doc = '''subtitle position''')
        
    prop_sub_alignment = property(
        lambda self: self.get_property("sub_alignment"),
        lambda self, val: self.set_property("sub_alignment", val),
        doc = '''subtitle alignment''')
        
    prop_sub_visibility = property(
        lambda self: self.get_property("sub_visibility"),
        lambda self, val: self.set_property("sub_visibility", val),
        doc = '''show/hide subtitles''')
        
    prop_sub_forced_only = property(
        lambda self: self.get_property("sub_forced_only"),
        lambda self, val: self.set_property("sub_forced_only", val),
        doc = None)
        
    prop_sub_scale = property(
        lambda self: self.get_property("sub_scale"),
        lambda self, val: self.set_property("sub_scale", val),
        doc = '''subtitles font size''')
        
    prop_tv_brightness = property(
        lambda self: self.get_property("tv_brightness"),
        lambda self, val: self.set_property("tv_brightness", val),
        doc = None)
        
    prop_tv_contrast = property(
        lambda self: self.get_property("tv_contrast"),
        lambda self, val: self.set_property("tv_contrast", val),
        doc = None)
        
    prop_tv_saturation = property(
        lambda self: self.get_property("tv_saturation"),
        lambda self, val: self.set_property("tv_saturation", val),
        doc = None)
        
    prop_tv_hue = property(
        lambda self: self.get_property("tv_hue"),
        lambda self, val: self.set_property("tv_hue", val),
        doc = None)
        
    prop_teletext_page = property(
        lambda self: self.get_property("teletext_page"),
        lambda self, val: self.set_property("teletext_page", val),
        doc = None)
        
    prop_teletext_subpage = property(
        lambda self: self.get_property("teletext_subpage"),
        lambda self, val: self.set_property("teletext_subpage", val),
        doc = None)
        
    prop_teletext_mode = property(
        lambda self: self.get_property("teletext_mode"),
        lambda self, val: self.set_property("teletext_mode", val),
        doc = '''0 - off, 1 - on''')
        
    prop_teletext_format = property(
        lambda self: self.get_property("teletext_format"),
        lambda self, val: self.set_property("teletext_format", val),
        doc = '''0 - opaque,
1 - transparent,
2 - opaque inverted,
3 - transp. inv.''')
        



__all__ = [
    'do_play', 'do_pause',
    'do_next', 'do_prev', 'do_shuffle',
    'do_infos', 'do_playlist',
    'do_tag', 'do_rate',
]

import urllib
from zicbee.core.zshell import args
from zicbee.core.config import config
from .search import do_search

def do_play(dbhost=config.db_host, phost=config.player_host):
    """ Play the specified pattern, same syntax as "search". """
    play_uri = 'http://%s/search?id=&host=%s&pattern=%s'%(phost, dbhost, urllib.quote(u' '.join(args)))
    urllib.urlopen(play_uri).read()

def do_infos(host=config.player_host):
    """ Show informations about currently playing song """
    play_uri = 'http://%s/infos?fmt=txt'%(host)
    site = urllib.urlopen(play_uri)
    while True:
        l = site.readline()
        if not l:
            break
        print l,

def do_playlist(host=config.player_host):
    """ Show current playing list """
    play_uri = 'http://%s/playlist?fmt=txt'%(host)
    site = urllib.urlopen(play_uri)
    while True:
        l = site.readline()
        if not l:
            break
        print l,

def do_pause(host=config.player_host):
    """ Toggle pause on player """
    play_uri = 'http://%s/pause'%(host)
    urllib.urlopen(play_uri).read()

def do_shuffle(host=config.player_host):
    """ Shuffles the playing list (results in a random playlist) """
    play_uri = 'http://%s/shuffle'%(host)
    urllib.urlopen(play_uri).read()

def do_next(host=config.player_host):
    """ Switch to next track """
    play_uri = 'http://%s/next'%(host)
    urllib.urlopen(play_uri).read()

def do_prev(host=config.player_host):
    """ Switch to previous track """
    play_uri = 'http://%s/prev'%(host)
    urllib.urlopen(play_uri).read()

def do_tag(tag, host=config.player_host):
    """ Tag selected pattern with specified rating.
    options:
        tag
        host=<db_host>
        + same pattern as "search"
    ex:
        tag::jazz,funny artist: richard cheese
        tag::rock artist: noir d
    Note multi-tagging is allowed by using "," separator (NO BLANK!)
(EXPERIMENTAL)
    """
    def song_rater(song):
        uri = song[0]
        sid = (song[0].rsplit('=', 1)[1])
        rate_uri = uri[:uri.index('/db/')+3] + '/tag/%s/%s'%(sid, tag)
        urllib.urlopen(rate_uri)

    do_search(out=song_rater, host=host, edit_mode=True)

def do_rate(rate=1, host=config.db_host):
    """ Rate selected pattern with specified rating.
    options:
        rate=1
        host=<db_host>
        + same pattern as "search"
    ex:
        rate::3:guntah.myhost.com artist: Brassens
        rate::0 title: Very bad song artist: very bad artist
(EXPERIMENTAL) """
    def song_rater(song):
        uri = song[0]
        sid = (song[0].rsplit('=', 1)[1])
        rate_uri = uri[:uri.index('/db/')+3] + '/rate/%s/%s'%(sid, rate)
        urllib.urlopen(rate_uri)

    do_search(out=song_rater, host=host, edit_mode=True)




import gobject
gobject.threads_init()

from zicbee.core.zutils import DEBUG
from zicbee.core.debug import log

class DelayedAction(object):
    def __init__(self, fn, *args, **kw):
        """ You may pass _prio & _delay as keyword arguments
        """
        self.fn = fn
        self.running = None

        self._prio = kw.get('_prio', gobject.PRIORITY_DEFAULT_IDLE)
        try:
            del kw['_prio']
        except KeyError:
            pass

        self._delay = int(kw.get('_delay', 1) * 1000)
        try:
            del kw['_delay']
        except KeyError:
            pass

        self.args = tuple(args)
        self.kw = kw

    def _run(self, *args):
        try:
            self.fn(*self.args, **self.kw)
            self.running = None
        except Exception, e:
            DEBUG()
        return False

    def start(self, delay=None, prio=None, args=None, kwargs=None):
        """ (Re)Start the action
        if delay and/or prio are not specified, last used are used
        """
        """ start action after 'delay' seconds. """

        if delay is not None:
            self._delay = int(delay * 1000)

        if prio is not None:
            self._prio = prio

        if args is not None:
            self.args = args

        if kwargs is not None:
            self.kwargs = kwargs

        self.stop()
        self.running = gobject.timeout_add(self._delay, self._run,
                priority=self._prio)

    def stop(self):
        """ Stop the action if running """
        if self.running is not None:
            gobject.source_remove(self.running)
        self.running = None

class IterableAction(object):
    def __init__(self, it):
        self.it = it
        self.running = None
        self._delay = 1

    def _run(self, *args):
        if self.running is None:
            return False
        try:
            self.it.next()
        except StopIteration:
            self.stop()
        except:
            DEBUG()
            log.error('STOPPING %r!', self)
        else:
            return True
        return False

    def __repr__(self):
        return "<IterableAction %s %srunning/%s>"%(self.it, '' if self.running else 'not ', self._delay)

    def start(self, delay=0, prio=gobject.PRIORITY_DEFAULT_IDLE):
        """ start action after 'delay' seconds. """
        self.stop()
        self.running = gobject.timeout_add(int(delay*1000), self._run, priority=prio)
        return self

    def start_on_fd(self, fd, prio=gobject.PRIORITY_DEFAULT_IDLE):
        """ start action if 'fd' is readable. """
        self.stop()
        self.running = gobject.io_add_watch(fd, gobject.IO_IN, self._run, priority=prio)
        return self

    def stop(self):
        if self.running is not None:
            gobject.source_remove(self.running)
        self.running = None




# vim: et ts=4 sw=4
from __future__ import with_statement

import os
import random # shuffle
import thread
import web
import urllib
import difflib
from time import sleep
from pkg_resources import resource_filename
from threading import RLock
import itertools
from time import time
from zicbee.core.zshell import songs
from zicbee.core.zutils import compact_int, jdump, jload, parse_line
from zicbee.core.zutils import uncompact_int, DEBUG
from zicbee.core.config import config

import logging
log = logging.getLogger()

web.config.debug = True if config.debug and str(config.debug).lower() in ('on', 'yes') else False

WEB_FIELDS = 'artist album title length score tags'.split() + ['__id__']

web.internalerror = web.debugerror

# Set default headers & go to templates directory
web.ctx.headers = [('Content-Type', 'text/html; charset=utf-8'), ('Expires', 'Thu, 01 Dec 1994 16:00:00 GMT')]
render = web.template.render(resource_filename('zicbee.ui.web', 'web_templates'))

SimpleSearchForm = web.form.Form(
        web.form.Hidden('id'),
        web.form.Textbox('host', description='Search host', value='localhost'),
        web.form.Textbox('pattern', description='Search pattern'),
#        web.form.Textbox('tempname', description='Temporary name'),
        )

TagForm = web.form.Form(web.form.Textbox('tag', description='Set tag'))
ScoreForm = web.form.Form(web.form.Dropdown('score', range(11), description='Set rate'))

# DB part

db_render = web.template.render(resource_filename('zicbee.ui.web', 'web_templates'))

DbSimpleSearchForm = web.form.Form(
        web.form.Hidden('id'),
        web.form.Textbox('pattern'),
        web.form.Checkbox('m3u'),
        )

class PlayerCtl(object):
    """ The player interface, this should lead to a constant code, with an interchangeable backend
    See self.player.* for the needed interface.
    """
    def __init__(self):
        self._cur_song_pos = -1
        self.playlist = []
        self.views = []
        try:
            from . import mp # mplayer backend
            self.player = mp.MPlayer()
        except:
            # TODO: add different backends!
            # (maybe adapt mp interface to be more generic)
            raise RuntimeError("No backend could be loaded!")
        self.position = None
        self._lock = RLock()
        self._paused = False
        thread.start_new_thread(self._main_loop, tuple())
        self._named_playlists = dict()

    def __repr__(self):
        return '<Player[%d] playing %s (views=%d)>'%(len(self.playlist), self.selected, len(self.views))

    def _main_loop(self):
        while True:
            if not self._paused and len(self.playlist):
                try:
                    try:
                        with self._lock:
                            self.position = int(self.player.prop_stream_pos/10000)
                    except IOError:
                        self.position = None
                        # restart player
                        self.player.wait()
                        self.player = mp.MPlayer()
                    except:
                        self.position = None

                    web.debug('pos: %s'%self.position)

                    if self.position is None:
                        i = self.select(1)
                        while True:
                            try:
                                with self._lock:
                                    i.next()
                            except StopIteration:
                                break
                except Exception, e:
                    web.debug('E: %s'%e)
                except:
                    web.debug('E: ???')
            sleep(1)

    def select(self, sense):
        """ Selects a song, according to the given offset
        ex.: self.select(1) # selects the next track
        self.select(-1) # selects the previous track
        self.select(0) # no-op
        """

        with self._lock:
            pos = self._cur_song_pos
            self._cur_song_pos += sense
            web.debug(self.infos, self.selected)

            if self._cur_song_pos > len(self.playlist):
                self._cur_song_pos = -1
            elif self._cur_song_pos < -1:
                self._cur_song_pos = -1

            song_name = config.streaming_file
            web.debug('download: %s'%self.selected_uri)
            dl_it = self._download_zic(self.selected_uri, song_name)
            dl_it.next()
            web.debug('select: %d'%self._cur_song_pos)
            if pos != self._cur_song_pos:
                web.debug("Loadfile %d/%s : %s !!"%(self._cur_song_pos, len(self.playlist), song_name))
                self.player.loadfile(song_name)
            self._paused = False
        return dl_it

    def tag(self, tag):
        ci = compact_int(self.selected['__id__'])
        uri = 'http://%s/db/tag/%s/%s'%(self.hostname, ci, tag)
        urllib.urlopen(uri)

    def rate(self, score):
        web.debug('DEBUG: %s'%self.selected)
        ci = compact_int(self.selected['__id__'])
        uri = 'http://%s/db/rate/%s/%s'%(self.hostname, ci, score)
        web.debug('URI: %s'%uri)
        urllib.urlopen(uri)

    def shuffle(self):
        """ Shuffle the playlist, and selects the first track
        if the playlist is empty, do nothing
        """
        if len(self.playlist) == 0:
            return
        with self._lock:
            random.shuffle(self.playlist)
            self._cur_song_pos = 0

    def seek(self, val):
        """ Seek according to given value
        """
        with self._lock:
            self.player.seek(val)

    def pause(self):
        """ (Un)Pause the player
        """
        with self._lock:
            self.player.pause()
            self._paused = not self._paused

    def delete_playlist(self, name):
        """ Delete the given named playlist (by name) """
        del self._named_playlists[name]

    def delete_entry(self, position):
        """ delete the song at the given position """
        del self.playlist[position]

    def move_entry(self, pos1, pos2):
        """ Swap two entries in the current playlist """
        p = self.playlist
        p[pos1], p[pos2] = p[pos2], p[pos1]

    def playlist_change(self, operation, pls_name):
        """ copy or append a named playlist to the active one
        """
        if operation == 'copy':
            self.playlist = self._named_playlists[pls_name]
            self._cur_song_pos = 0
        if operation == 'append':
            self.playlist.extend(self._named_playlists[pls_name])

    def fetch_playlist(self, hostname=None, temp=False, **kw):
        """
        Fetch a playlist from a given hostname,
        can take any keyword, will be passed with the remote command
        some useful keywords:
            pattern: a search string
            db: the database name (default is ok in general)
        if the temp keyword is given,
        a temporary playlist will be created with the given name
        (the main one is not affected)
        returns an iterator
        """
        web.debug('fetch_pl: h=%s tmp=%s'%(hostname, temp))
        hostname = hostname.strip()
        if not hostname:
            hostname = '127.0.0.1'

        if ':' not in hostname:
            hostname += ':9090'

        with self._lock:
            self.hostname = hostname

            uri = 'http://%s/db/?json=1&%s'%(hostname, urllib.urlencode(kw))
            site = urllib.urlopen(uri)
            web.debug('fetch_pl: kw=%s uri=%s'%(kw, uri))
            if temp:
                self._named_playlists[temp] = []
                add = self._named_playlists[temp]
            else:
                self.playlist[:] = []
                add = self.playlist.append

        total = 0
        done = False

        while True:
            for n in xrange(50):
                line = site.readline()
                if not line:
                    break
                try:
                    r = jload(line)
                except:
                    web.debug("Can't load json description: %s"%line)
                    break
                total += r[4]
                web.debug('r=%s'%r)
                with self._lock:
                    add(r)
                self.signal_view('update_total', total)

            yield
            if not line:
                break

        if temp:
            self._total_length = total
        else:
            # reset song position
            with self._lock:
                self._cur_song_pos = 0
                self._tmp_total_length = total

    def _download_zic(self, uri, fname):
        if getattr(self, '_download_stream', None):
            self._download_stream.close()

        fd = file(fname, 'wb')
        self._download_stream = fd

        site = urllib.urlopen(uri)
        total = float(site.info().getheader('Content-Length'))
        total_length = 100
        achieved = 0

        data = site.read(2**17)
        fd.write(data) # read ~130k (a few seconds in general)
        achieved += len(data)
        self._running = True
        yield site.fileno()
#        web.debug('downloading %d'%achieved)

        try:
            BUF_SZ = 2**14 # 16k micro chunks
            while True:
                data = site.read(BUF_SZ)
                progress = total_length * (achieved / total)
#                self.signal_view('download_progress', progress)
                if not data:
                    break
                achieved += len(data)
                web.debug('downloading %s from %s (%.1f%%)'%(uri, self, progress))
                fd.write(data)
                yield
            fd.close()
        except Exception, e:
            web.debug('ERROR %s %s'%(repr(e), dir(e)))
        finally:
            if fd is self._download_stream:
                self._download_stream = None

    def signal_view(self, name, *args, **kw):
        web.debug('signaling %s %s %s'%(name, args, kw))
        for view in self.views:
            web.debug(' -> to view %s'%view)
            try:
                getattr(view, 'SIG_'+name)(*args, **kw)
            except:
                DEBUG()

    def _get_infos(self, l):
        try:
            d = dict(zip(['uri']+WEB_FIELDS, l))
            d['length'] = int(d['length'])
            d['score'] = d['score'] or 0.0
            d['tags'] = d['tags'] or u''
            return d
        except Exception, e:
            web.debug('_get_infos ERR: %s'%e)
            return None

    @property
    def selected(self):
        pos = self._cur_song_pos
        if not (0 <= pos <= len(self.playlist)):
            return None
        else:
            return self._get_infos(self.playlist[pos])

    @property
    def selected_uri(self):
        try:
            with self._lock:
                if self._cur_song_pos < 0:
                    raise Exception()
                txt =  self.playlist[self._cur_song_pos][0]
            return txt
        except Exception, e:
            web.debug("selected_uri ERR: %s"%e)
            return None

    @property
    def infos(self):
        return dict(
                current = self._cur_song_pos,
                total = len(self.playlist),
                running = self._cur_song_pos >= 0,
                )

class webplayer:
    player = PlayerCtl()

    GET = web.autodelegate('REQ_')
    lastlog = []

    def REQ_main(self):
        web.debug('coin')
        cook_jar = web.cookies(host='localhost', pattern='')
        cook_jar['pattern'] = urllib.unquote(cook_jar['pattern'])
        af = SimpleSearchForm(True)
        sf = ScoreForm(True)
        tf = TagForm(True)
        af.fill(cook_jar)
        yield unicode(render.player(af, sf, tf))

    REQ_ = REQ_main # default page

    def REQ_search(self):
        it = None
        try:
            i = web.input()
            if i.get('pattern'):
                it = self.player.fetch_playlist(i.get('host', 'localhost'), pattern=i.pattern, temp=i.get('tempname', '').strip() or False)
            else:
                it = self.player.fetch_playlist(i.get('host', 'localhost'), i.get('tempname', '').strip() or False)
            it.next()

        except (IndexError, KeyError):
            it = None
        finally:
            return itertools.chain(it, [web.redirect('/')])

    def REQ_delete(self):
        i = web.input()
        self.player.delete_entry(int(i['idx']))
        return web.redirect('/')

    def REQ_move(self):
        i = web.input()
        self.player.move_entry(int(i['i1']), int(i['i2']))
        return web.redirect('/')

    def REQ_append(self):
        self.player.playlist_change('append', web.input()['name'])
        return web.redirect('/')

    def REQ_copy(self):
        self.player.playlist_change('copy', web.input()['name'])
        return web.redirect('/')

    def REQ_infos(self):
        i = web.input()
        format = i.get('fmt', 'txt')

        try:
            _d = self.player.selected.copy()
            _d['uri'] = self.player.playlist[self.player._cur_song_pos][0]
        except AttributeError:
            _d = dict()
        _d['pls_position'] = self.player._cur_song_pos
        _d['song_position'] = self.player.position
        _d['pls_size'] = len(self.player.playlist)

        try:
            _d['id'] = compact_int(_d.pop('__id__'))
        except KeyError:
            pass

        if format == 'txt':
            for k, v in _d.iteritems():
                yield '%s: %s\n'%(k, v)
        elif format == 'json':
            yield jdump(_d)

    def REQ_lastlog(self):
        return '\n'.join(self.lastlog)

    def REQ_playlist(self):
        i = web.input()
        pls = self.player.playlist

        start = int(i.get('start', 0))

        format = i.get('fmt', 'txt')

        if i.get('res'):
            end = start + int(i.res)
        else:
            end = len(pls)

        window_iterator = (pls[i] + [i] for i in xrange(start, min(len(pls), end)))

        if format == 'txt':
            for e in window_iterator:
                yield '%s\n'%(' , '.join(str(t) for t in e))
        elif format == 'json':
            yield jdump(list(window_iterator))

    def REQ_guess(self, guess):
        try:
            self.player.selected.iteritems
        except AttributeError:
            yield jdump(False)
            return

        artist = self.player.selected['artist']
        title = self.player.selected['title']
        if difflib.get_close_matches(guess, (artist, title)):
            yield jdump(True)
        else:
            yield jdump(False)

    def REQ_shuffle(self):
        return self.player.shuffle()

    def REQ_pause(self):
        return self.player.pause()

    def REQ_prev(self):
        return self.player.select(-1)

    def REQ_next(self):
        return self.player.select(1)

    def REQ_tag(self, tag):
        return self.player.tag(unicode(tag.lstrip('/')))

    def REQ_rate(self, score):
        return self.player.rate(score.lstrip('/'))

    def REQ_seek(self, val):
        val = val[1:]
        web.debug('VAL=%s'%val)
#        self.player.seek(int(val))

def refresh_db():
    songs.db.commit()
    songs._init()
    songs.db.cleanup()

class web_db_index:

    _db_lock = RLock()

    def tag(self, song, tag):

        song_id = uncompact_int(song)
        try:
            tag = unicode(tag)
            with self._db_lock:
                SEP=u','
                _t = songs[song_id].tags
                tag_set = set( _t.strip(SEP).split(SEP) ) if _t else set()
                for single_tag in tag.split(','):
                    tag_set.add(single_tag.strip())
                new_tag = ''.join((SEP,SEP.join(tag_set),SEP))
                songs[song_id].update(tags=new_tag)
        except Exception, e:
            web.debug('E!%s'%e)
        finally:
            refresh_db()

    def rate(self, song, rating):
        web.debug('rate: song=%s rating=%s'%(song,rating))
        try:
            with self._db_lock:
                song_id = uncompact_int(song)
                songs[song_id].update(score=int(rating))
        finally:
            refresh_db()

    def multirate(self, ratings_list):
        web.debug('rate: ratings_list=%s'%ratings_list)
        try:
            ratings = [rating.split('=') for rating in ratings_list.split(',')]
            with self._db_lock:
                for song, rating in ratings:
                        song_id = uncompact_int(song)
                        songs[song_id].update(score=int(rating))
        except:
            pass
        finally:
            refresh_db()

    def GET(self, name):
        hd = web.webapi.ctx.homedomain
        t0 = time()
        af = DbSimpleSearchForm()
        if name.startswith('rate/'):
            self.rate(*name.split('/', 3)[1:])
            return
        if name.startswith('multirate/'):
            self.multirate(name.split('/', 2)[1])
            return
        elif name.startswith('kill'):
            songs.db.close()
            raise SystemExit()
        elif name.startswith('tag'):
            self.tag(*name.split('/', 3)[1:])
            return
        elif af.validates():
            try:
                af.fill()
                song_id = af['id'].value
                if song_id:
                    song_id = uncompact_int(song_id)
                    if name.startswith("get"):
                        filename = songs[song_id].filename
                        web.header('Content-Type', 'application/x-audio')
                        web.header('Content-Disposition',
                                'attachment; filename:%s'%filename.rsplit('/', 1)[-1], unique=True)

                        CHUNK=1024
                        in_fd = file(filename)
                        web.header('Content-Length', str( os.fstat(in_fd.fileno()).st_size ) )
                        yield

                        while True:
                            data = in_fd.read(CHUNK)
                            if not data: break
                            y = (yield data)
                        return
                    else:
                        song = songs[song_id]
                        for f in song.fields:
                            yield "<b>%s</b>: %s<br/>"%(f, getattr(song, f))
                        return
            except GeneratorExit:
                raise
            except Exception, e:
                web.debug(e)

        if af['m3u'].value:
            web.header('Content-Type', 'audio/x-mpegurl')
            format = 'm3u'
        elif web.input().get('plain'):
            format = 'plain'
        elif web.input().get('json'):
            format = 'json'
        else:
            web.header('Content-Type', 'text/html; charset=utf-8')
            format = 'html'

        pattern = af['pattern'].value

        if pattern is None:
            res = None
        else:
            pat, vars = parse_line(pattern)
            urlencode = web.http.urlencode
            ci = compact_int
            web.debug('searching %s %s...'%(pat, vars))

            res = ([hd+'/db/get/%s?id=%s'%('song'+r.filename[-4:], ci(int(r.__id__))), r]
                    for r in songs.search(list(WEB_FIELDS)+['filename'], pat, **vars)
                    )
        t_sel = time()

        if format == 'm3u':
            yield unicode(db_render.m3u(web.http.url, res))
        elif format == 'plain':
            yield unicode(db_render.plain(af, web.http.url, res))
        elif format == 'json':
            # try to pre-compute useful things
            field_decoder = zip( WEB_FIELDS,
                    (songs.db.f_decode[songs.db.fields[fname]] for fname in WEB_FIELDS)
                    )
            yield

            infos_iterator = ( [r[0]] + [d(r[1][r[1].fields.index(f)]) for f, d in field_decoder]
                    for r in res )
            try:
                # TODO: optimise this (jdump by 100 size blocks)
                for r in infos_iterator:
                    yield jdump(r)
                    yield '\n'
                web.debug('handled in %.2fs (%.2f for select)'%(time() - t0, t_sel - t0))
            except Exception, e:
                web.debug("ERR:", e)
        else:
            yield unicode(db_render.index(af, res))



from time import time
import sys
from zicbee.db import valid_tags
from zicbee.core.zshell import args, songs
from zicbee.core.zutils import duration_tidy, parse_line, jload
from zicbee.core.config import config

def do_search(out=None, host=config.db_host, edit_mode=False):
    """ Search for song, display results.
    See "help" for a more complete documentation
    """
    if ':' not in host:
        host += ':9090'

    duration = 0
    start_t = time()

    fields = list(valid_tags)
    fields.remove('filename')
    fields = tuple(fields)

    if callable(out):
        song_output = out
    elif out == 'm3u':
        def song_output(song):
            print song[0]
    elif out == 'null':
        def song_output(song): pass
    else:
        def song_output(song):
            txt = '%s :\n%s [%s, score: %s, tags: %s]'%(song[0],
                    ' - '.join(str(i) for i in song[1:4]),
                    duration_tidy(song[4]), song[5], song[6],
                    )
            print txt.decode('utf8').encode('utf8')

    num = 0
    if host is not None:
        import urllib
        params = {'pattern':' '.join(args)}
        uri = 'http://%s/db/?json=1&%s'%(host, urllib.urlencode(params))
        site = urllib.urlopen(uri)
        while True:
            line = site.readline()
            if not line:
                break
            r = jload(line)
            song_output(r)
            duration += r[4]
            num += 1
    else:
        pat, kw = parse_line(' '.join(args))

        if edit_mode:
            search_fn = songs.u_search
        else:
            search_fn = songs.search

        for num, res in enumerate(search_fn(None, pat, **kw)):
            song_output(res)
            duration += res.length

    sys.stderr.write("# %d results in %s for a total of %s!\n"%(
            num,
            duration_tidy(time()-start_t),
            duration_tidy(duration)
            ))





import os
import atexit
import ConfigParser

__all__ = ['DB_DIR', 'defaults_dict', 'config']

DB_DIR = os.path.expanduser(os.getenv('ZICDB_PATH') or '~/.zicdb')
try: # Ensure personal dir exists
    os.mkdir(DB_DIR)
except:
    pass

defaults_dict = {
        'streaming_file' : '/tmp/zsong',
        'download_dir' : '/tmp',
        'db_host' : 'localhost:9090',
        'player_host' : 'localhost:9090',
        'debug' : '',
        'default_search' : '',
        'web_skin' : '',
        }

config_filename = os.path.join(DB_DIR, 'config.ini')

class ConfigObj(object):

    _cfg = ConfigParser.ConfigParser(defaults_dict)

    def __init__(self):
        if os.path.exists(config_filename):
            self._cfg.read(config_filename)
        else:
            self._cfg.write(file(config_filename, 'w'))

    def __setattr__(self, name, val):
        return self._cfg.set('DEFAULT', name, val)

    def __getattr__(self, name):
        return self._cfg.get('DEFAULT', name)

# Ensure the file is written on drive
atexit.register(lambda: config._cfg.write(file(config_filename, 'w')))

config = ConfigObj()

if config.debug:
    import logging
    log = logging.getLogger()
    log.setLevel(logging.DEBUG)




import os
import logging
from logging import getLogger as get_log
log = get_log()

default_handler = logging.StreamHandler()
log.addHandler(default_handler)

default_formatter = logging.Formatter('[%(threadName)s %(relativeCreated)d] %(module)s %(funcName)s:%(lineno)s %(message)s')
default_handler.setFormatter( default_formatter )




#!/usr/bin/env python
import os
import sys
try:
	import setuptools
except ImportError:
	from ez_setup import use_setuptools
	use_setuptools()
from setuptools import setup, find_packages

VERSION='0.7'

if 'install' in sys.argv:
    print """Warning:
You will need to install some parts manually:

If it can't build, try to comment the line:
            'python-cjson>=1.0.5',

and uncomment one of those (recommended alternative: simplejson):
#            'simplejson>=1.7.3',
#            'demjson>=1.1',

Good luck !"""



# also supported:
#            'simplejson>=1.7.3',

requirements = [ 'buzhug>=1.4', 'mutagen>=1.14' ]

if os.name in ('nt', 'ce'):
    requirements.append( 'demjson>=1.1' )
else:
    requirements.append( 'python-cjson>=1.0.5' )

setup (
        name='zicbee',
        version=VERSION,
        author='Fabien Devaux',
        author_email='fdev31@gmail.com',
        license='BSD',
        platform='All',
        description='A simple & powerful distributed Music database engine',
        long_description='''
ZicBee is a project grouping multiple applications to manage play and handle music databases.

For now there is a Swiss-army knife tool: zicdb

Some plugins for quodlibet has also be developed. ZicBee is fast,
portable (but not very ported...) and flexible.

While the project is stable and usable (there online doc and a nice www gui),
it's mostly interesting for hackers and developers for now.

See features list, it's mostly handy for people with large databases,
with optionally multiple computers.
It can be adapted to handle video too, hacking some bit of code.
        ''',
        url='http://code.google.com/p/zicdb/',
        keywords='database music tags metadata management',
        packages = find_packages(),
        package_data = {
            'zicbee': ['ui/web/web_templates/*.html', 'ui/web/static/*.css', 'ui/web/static/*.css', 'ui/web/static/MochoKit/*.js', 'ui/gtk/*.glade'],
            },

        entry_points = {
            "console_scripts": [
                'zicdb = zicbee.core:startup',
                'zicserve = zicbee.core:serve [server]',
                'zicgui = zicbee.ui.gtk.player:main [player]'
                ],
            "setuptools.installation" : [
                'eggsecutable = zicbee.core:startup'
                ]
            },

        install_requires = requirements,

        extras_require = dict(
#            player='pyglet>=1.2',
            server='web.py>=0.31',
            ),

        dependency_links = [
            'eggs',
#            'http://code.google.com/p/pyglet/downloads/list',
            ],
        )

if 'build' in sys.argv or 'install' in sys.argv or any(a for a in sys.argv if 'dist' in a):
    dec = "*"*80
    print dec
    print "* WARNING"
    print dec
    print "* YOU NEED MPLAYER IN YOUR PATH TO GET PLAYER FEATURES"
    print dec




# vim: et ts=4 sw=4
import os
import sys
from zicbee.db import Database

DEFAULT_NAME='songs'

def init(args=None):
    clean_args = args or sys.argv[2:]
    db = Database(os.environ.get('ZDB', DEFAULT_NAME))
    globals().update( dict(songs=db, args=clean_args) )
    db.db.cleanup() # XXX: Ugly !
